import { __decorate, __metadata } from 'tslib';
import deepEqual from 'fast-deep-equal';
import { ElementRef, Input, Component, NgModule } from '@angular/core';
import { Calendar } from '@fullcalendar/core';

const hasOwnProperty = Object.prototype.hasOwnProperty;
/*
Really simple clone utility. Only copies plain arrays, objects, and Dates. Transfers everything else as-is.
Wanted to use a third-party lib, but none did exactly this.
*/
function deepCopy(input) {
    if (Array.isArray(input)) {
        return input.map(deepCopy);
    }
    else if (input instanceof Date) {
        return new Date(input.valueOf());
    }
    else if (typeof input === 'object' && input) { // non-null object
        return mapHash(input, deepCopy);
    }
    else { // everything else (null, function, etc)
        return input;
    }
}
function shallowCopy(val) {
    if (typeof val === 'object') {
        if (Array.isArray(val)) {
            val = Array.prototype.slice.call(val);
        }
        else if (val) { // non-null
            val = Object.assign({}, val);
        }
    }
    return val;
}
function mapHash(input, func) {
    const output = {};
    for (const key in input) {
        if (hasOwnProperty.call(input, key)) {
            output[key] = func(input[key], key);
        }
    }
    return output;
}

const OPTION_IS_DEEP = {
    header: true,
    footer: true,
    events: true,
    eventSources: true,
    resources: true
};

let FullCalendarComponent = class FullCalendarComponent {
    constructor(element) {
        this.element = element;
        this.optionSnapshot = {}; // for diffing only
    }
    ngAfterViewInit() {
        const { deepChangeDetection } = this;
        const options = this.options || {};
        // initialize snapshot
        this.optionSnapshot = mapHash(options, (optionVal, optionName) => ((deepChangeDetection && OPTION_IS_DEEP[optionName])
            ? deepCopy(optionVal)
            : optionVal));
        this.calendar = new Calendar(this.element.nativeElement, options);
        this.calendar.render();
    }
    /*
    allows us to manually detect complex input changes, internal mutations to certain options.
    called before ngOnChanges. called much more often than ngOnChanges.
    */
    ngDoCheck() {
        if (this.calendar) { // not the initial render
            const { deepChangeDetection, optionSnapshot } = this;
            const options = this.options || {};
            const updates = {};
            const removals = [];
            let anyChanges = false;
            // detect adds and updates (and update snapshot)
            for (const optionName in options) {
                if (options.hasOwnProperty(optionName)) {
                    const optionVal = options[optionName];
                    if (deepChangeDetection && OPTION_IS_DEEP[optionName]) {
                        if (!deepEqual(optionSnapshot[optionName], optionVal)) {
                            optionSnapshot[optionName] = deepCopy(optionVal);
                            // trick FC into knowing about a nested change.
                            // TODO: future versions won't need this.
                            // can't use the previously-made deep copy because it blows away prototype-association.
                            updates[optionName] = shallowCopy(optionVal);
                            anyChanges = true;
                        }
                    }
                    else {
                        if (optionSnapshot[optionName] !== optionVal) {
                            optionSnapshot[optionName] = optionVal;
                            updates[optionName] = optionVal;
                            anyChanges = true;
                        }
                    }
                }
            }
            const oldOptionNames = Object.keys(optionSnapshot);
            // detect removals (and update snapshot)
            for (const optionName of oldOptionNames) {
                if (!(optionName in options)) { // doesn't exist in new options?
                    delete optionSnapshot[optionName];
                    removals.push(optionName);
                }
            }
            if (anyChanges) {
                this.calendar.pauseRendering();
                this.calendar.mutateOptions(updates, removals);
            }
        }
    }
    ngAfterContentChecked() {
        if (this.calendar) { // too defensive?
            this.calendar.resumeRendering();
        }
    }
    ngOnDestroy() {
        if (this.calendar) { // too defensive?
            this.calendar.destroy();
            this.calendar = null;
        }
    }
    getApi() {
        return this.calendar;
    }
};
FullCalendarComponent.ctorParameters = () => [
    { type: ElementRef }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], FullCalendarComponent.prototype, "options", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], FullCalendarComponent.prototype, "deepChangeDetection", void 0);
FullCalendarComponent = __decorate([
    Component({
        selector: 'full-calendar',
        template: ''
    }),
    __metadata("design:paramtypes", [ElementRef])
], FullCalendarComponent);

let FullCalendarModule = class FullCalendarModule {
};
FullCalendarModule = __decorate([
    NgModule({
        declarations: [FullCalendarComponent],
        imports: [],
        exports: [FullCalendarComponent]
    })
], FullCalendarModule);

/*
 * Public API Surface of fullcalendar
 */

/**
 * Generated bundle index. Do not edit.
 */

export { FullCalendarComponent, FullCalendarModule };
//# sourceMappingURL=fullcalendar-angular.js.map
