import { __assign, __values, __decorate, __metadata } from 'tslib';
import deepEqual from 'fast-deep-equal';
import { ElementRef, Input, Component, NgModule } from '@angular/core';
import { Calendar } from '@fullcalendar/core';

var hasOwnProperty = Object.prototype.hasOwnProperty;
/*
Really simple clone utility. Only copies plain arrays, objects, and Dates. Transfers everything else as-is.
Wanted to use a third-party lib, but none did exactly this.
*/
function deepCopy(input) {
    if (Array.isArray(input)) {
        return input.map(deepCopy);
    }
    else if (input instanceof Date) {
        return new Date(input.valueOf());
    }
    else if (typeof input === 'object' && input) { // non-null object
        return mapHash(input, deepCopy);
    }
    else { // everything else (null, function, etc)
        return input;
    }
}
function shallowCopy(val) {
    if (typeof val === 'object') {
        if (Array.isArray(val)) {
            val = Array.prototype.slice.call(val);
        }
        else if (val) { // non-null
            val = __assign({}, val);
        }
    }
    return val;
}
function mapHash(input, func) {
    var output = {};
    for (var key in input) {
        if (hasOwnProperty.call(input, key)) {
            output[key] = func(input[key], key);
        }
    }
    return output;
}

var OPTION_IS_DEEP = {
    header: true,
    footer: true,
    events: true,
    eventSources: true,
    resources: true
};

var FullCalendarComponent = /** @class */ (function () {
    function FullCalendarComponent(element) {
        this.element = element;
        this.optionSnapshot = {}; // for diffing only
    }
    FullCalendarComponent.prototype.ngAfterViewInit = function () {
        var deepChangeDetection = this.deepChangeDetection;
        var options = this.options || {};
        // initialize snapshot
        this.optionSnapshot = mapHash(options, function (optionVal, optionName) { return ((deepChangeDetection && OPTION_IS_DEEP[optionName])
            ? deepCopy(optionVal)
            : optionVal); });
        this.calendar = new Calendar(this.element.nativeElement, options);
        this.calendar.render();
    };
    /*
    allows us to manually detect complex input changes, internal mutations to certain options.
    called before ngOnChanges. called much more often than ngOnChanges.
    */
    FullCalendarComponent.prototype.ngDoCheck = function () {
        var e_1, _a;
        if (this.calendar) { // not the initial render
            var _b = this, deepChangeDetection = _b.deepChangeDetection, optionSnapshot = _b.optionSnapshot;
            var options = this.options || {};
            var updates = {};
            var removals = [];
            var anyChanges = false;
            // detect adds and updates (and update snapshot)
            for (var optionName in options) {
                if (options.hasOwnProperty(optionName)) {
                    var optionVal = options[optionName];
                    if (deepChangeDetection && OPTION_IS_DEEP[optionName]) {
                        if (!deepEqual(optionSnapshot[optionName], optionVal)) {
                            optionSnapshot[optionName] = deepCopy(optionVal);
                            // trick FC into knowing about a nested change.
                            // TODO: future versions won't need this.
                            // can't use the previously-made deep copy because it blows away prototype-association.
                            updates[optionName] = shallowCopy(optionVal);
                            anyChanges = true;
                        }
                    }
                    else {
                        if (optionSnapshot[optionName] !== optionVal) {
                            optionSnapshot[optionName] = optionVal;
                            updates[optionName] = optionVal;
                            anyChanges = true;
                        }
                    }
                }
            }
            var oldOptionNames = Object.keys(optionSnapshot);
            try {
                // detect removals (and update snapshot)
                for (var oldOptionNames_1 = __values(oldOptionNames), oldOptionNames_1_1 = oldOptionNames_1.next(); !oldOptionNames_1_1.done; oldOptionNames_1_1 = oldOptionNames_1.next()) {
                    var optionName = oldOptionNames_1_1.value;
                    if (!(optionName in options)) { // doesn't exist in new options?
                        delete optionSnapshot[optionName];
                        removals.push(optionName);
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (oldOptionNames_1_1 && !oldOptionNames_1_1.done && (_a = oldOptionNames_1.return)) _a.call(oldOptionNames_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            if (anyChanges) {
                this.calendar.pauseRendering();
                this.calendar.mutateOptions(updates, removals);
            }
        }
    };
    FullCalendarComponent.prototype.ngAfterContentChecked = function () {
        if (this.calendar) { // too defensive?
            this.calendar.resumeRendering();
        }
    };
    FullCalendarComponent.prototype.ngOnDestroy = function () {
        if (this.calendar) { // too defensive?
            this.calendar.destroy();
            this.calendar = null;
        }
    };
    FullCalendarComponent.prototype.getApi = function () {
        return this.calendar;
    };
    FullCalendarComponent.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], FullCalendarComponent.prototype, "options", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Boolean)
    ], FullCalendarComponent.prototype, "deepChangeDetection", void 0);
    FullCalendarComponent = __decorate([
        Component({
            selector: 'full-calendar',
            template: ''
        }),
        __metadata("design:paramtypes", [ElementRef])
    ], FullCalendarComponent);
    return FullCalendarComponent;
}());

var FullCalendarModule = /** @class */ (function () {
    function FullCalendarModule() {
    }
    FullCalendarModule = __decorate([
        NgModule({
            declarations: [FullCalendarComponent],
            imports: [],
            exports: [FullCalendarComponent]
        })
    ], FullCalendarModule);
    return FullCalendarModule;
}());

/*
 * Public API Surface of fullcalendar
 */

/**
 * Generated bundle index. Do not edit.
 */

export { FullCalendarComponent, FullCalendarModule };
//# sourceMappingURL=fullcalendar-angular.js.map
